package generate

import "os"
import "fmt"
import "os/exec"
import "testing"
import "path/filepath"

func testGenerateRun(test *testing.T, protocol *Protocol, imports string, testCase string) {
	// reset data directory
	dir := "test/generate-run"
	err := os.RemoveAll(dir)
	if err != nil { test.Fatal(err) }
	err = os.MkdirAll(dir, 0750)
	if err != nil { test.Fatal(err) }

	// open files
	sourceFile, err := os.Create(filepath.Join(dir, "protocol.go"))
	if err != nil { test.Fatal(err) }
	defer sourceFile.Close()
	mainFile, err := os.Create(filepath.Join(dir, "main.go"))
	if err != nil { test.Fatal(err) }
	defer mainFile.Close()

	// generate protocol
	generator := Generator {
		Output: sourceFile,
		PackageName: "main",
	}
	_, err = generator.Generate(protocol)
	if err != nil { test.Fatal(err) }

	// build static source files
	imports = `
		import "log"
		import "bytes"
		import "git.tebibyte.media/sashakoshka/hopp/tape"
		import tu "git.tebibyte.media/sashakoshka/hopp/internal/testutil"
		` + imports
	setup := `log.Println("*** BEGIN TEST CASE OUTPUT ***")`
	teardown := `log.Println("--- END TEST CASE OUTPUT ---")`
	static := `
		func testEncode(message Message, correct tu.Snake) {
			buffer := bytes.Buffer { }
			encoder := tape.NewEncoder(&buffer)
			n, err := message.Encode(encoder)
			if err != nil { log.Fatalf("at %d: %v\n", n, err) }
			encoder.Flush()
			got := buffer.Bytes()
			log.Printf("got:     [%s]", tu.HexBytes(got))
			log.Println("correct:", correct)
			if n != len(got) {
				log.Fatalf("n incorrect: %d != %d\n", n, len(got))
			}
			if ok, n := correct.Check(got); !ok {
				log.Fatalln("not equal at", n)
			}
		}
		`
	fmt.Fprintf(
		mainFile, "package main\n%s\nfunc main() {\n%s\n%s\n%s\n}\n%s",
		imports, setup, testCase, teardown, static)

	// build and run test
	command := exec.Command("go", "run", "./generate/test/generate-run")
	workingDirAbs, err := filepath.Abs("..")
	if err != nil { test.Fatal(err) }
	command.Dir = workingDirAbs
	command.Env = os.Environ()
	output, err := command.CombinedOutput()
	test.Logf("output of %v:\n%s", command, output)
	if err != nil { test.Fatal(err) }
}
